"use strict";

exports.__esModule = true;
exports.Consumer = void 0;

var _preact = _interopRequireDefault(require("preact"));

var _differs = require("../react/differs");

var _raf = require("../react/raf");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _inheritsLoose(subClass, superClass) { subClass.prototype = Object.create(superClass.prototype); subClass.prototype.constructor = subClass; subClass.__proto__ = superClass; }

var canUseDOM = !!(typeof window !== 'undefined' && window.document && window.document.createElement);

var Consumer =
/*#__PURE__*/
function (_Preact$Component) {
  _inheritsLoose(Consumer, _Preact$Component);

  function Consumer(props, _ref) {
    var _this;

    var atom = _ref.atom;
    _this = _Preact$Component.call(this) || this;
    _this.state = _this.map(atom.get(), props);
    _this.isPure = typeof props.pure === 'undefined' ? true : props.pure;
    _this.shouldObserve = typeof props.observe === 'undefined' ? canUseDOM : props.observe;
    _this.scheduleUpdate = props.sync ? function () {
      return _this.update();
    } : (0, _raf.raf)(function () {
      return _this.update();
    });
    return _this;
  }

  var _proto = Consumer.prototype;

  _proto.observe = function observe() {
    var _this2 = this;

    if (!this.shouldObserve) return;
    var atom = this.context.atom;
    this.unobserve && this.unobserve();
    this.unobserve = atom.observe(function () {
      _this2.cancelUpdate = _this2.scheduleUpdate();
    });
    this.observedAtom = atom;
    delete this.boundActions;
    delete this.boundActionsSpec;
  };

  _proto.componentWillUnmount = function componentWillUnmount() {
    this.unobserve && this.unobserve();
    this.cancelUpdate && this.cancelUpdate();
    delete this.unobserve;
    delete this.cancelUpdate;
    delete this.observedAtom;
    delete this.boundActions;
    delete this.boundActionsSpec;
  };

  _proto.shouldComponentUpdate = function shouldComponentUpdate(nextProps, nextState) {
    if (!this.isPure) return true; // if it's <Consumer> with dynamic children, shortcut the check

    if (this.props.children !== nextProps.children) {
      return true;
    } // our state is mappedProps, this is the main optimisation


    if ((0, _differs.differs)(this.state, nextState)) {
      return true;
    } // in connect() case don't need to diff further, no extra props


    if (this.props.originalProps) {
      this.cancelUpdate && this.cancelUpdate();
      return false;
    } // in <Consumer /> case we also diff props


    if ((0, _differs.differs)(this.props, nextProps)) {
      return true;
    }

    this.cancelUpdate && this.cancelUpdate();
  };

  _proto.componentWillReceiveProps = function componentWillReceiveProps(nextProps) {
    var atom = this.context.atom;
    var nextMappedProps = this.map(atom.get(), nextProps);
    this.setState(nextMappedProps);
  };

  _proto.componentDidUpdate = function componentDidUpdate() {
    this.cancelUpdate && this.cancelUpdate();
  };

  _proto.map = function map(state, props) {
    var originalProps = props.originalProps,
        map = props.map;
    return Object.assign({}, originalProps, map ? map(state, originalProps) : {});
  };

  _proto.update = function update() {
    var atom = this.context.atom;
    var nextMappedProps = this.map(atom.get(), this.props);
    this.setState(nextMappedProps);
  };

  _proto.bindActions = function bindActions(actions, dispatch, mappedProps) {
    if (!actions) return {
      dispatch: dispatch
    };
    if (typeof actions === 'function') return actions(dispatch, mappedProps);

    if (!this.boundActions || this.boundActionsSpec !== actions) {
      this.boundActionsSpec = actions;
      this.boundActions = actions.reduce(function (acc, action) {
        acc[action] = function (payload) {
          return dispatch(action, payload);
        };

        return acc;
      }, {});
    }

    return this.boundActions;
  };

  _proto.render = function render(_ref2, state, _ref3) {
    var actions = _ref2.actions,
        originalProps = _ref2.originalProps,
        _render = _ref2.render,
        children = _ref2.children;
    var atom = _ref3.atom;
    // do this in render, because:
    //  doing in constructor would cause memory leaks in SSR
    //  doing in componentDidMount leads to the wrong order of subscriptions
    //  we don't have another hook in Preact's context to check if atom was swapped
    if (!this.unobserve || this.observedAtom !== this.context.atom) this.observe();
    var mappedProps = this.state;
    var boundActions = this.bindActions(actions, atom.dispatch, mappedProps);
    return (_render || children[0])(Object.assign({}, originalProps, mappedProps, boundActions));
  };

  return Consumer;
}(_preact.default.Component);

exports.Consumer = Consumer;