"use strict";

exports.__esModule = true;
exports.useStore = useStore;

var _react = require("react");

var _core = require("../core");

/**
 * A component local store.
 * A great complement to the global store.
 */
function useStore(setup, deps) {
  var config = (0, _react.useMemo)(function () {
    if (typeof setup === 'function') {
      return setup();
    } else {
      return null;
    }
  }, deps || []);

  var _useState = (0, _react.useState)(function () {
    return config ? (0, _core.createStore)(config) : setup;
  }),
      atom = _useState[0];

  var _useState2 = (0, _react.useState)(function () {
    return atom.actions;
  }),
      actions = _useState2[0],
      setActions = _useState2[1];

  var _useState3 = (0, _react.useState)(function () {
    return atom.get();
  }),
      state = _useState3[0],
      setState = _useState3[1];

  (0, _react.useEffect)(function () {
    var didUnmount = false;
    var unobserve = atom.observe(function () {
      if (!didUnmount) {
        setState(atom.get());
      }
    });
    return function () {
      didUnmount = true;
      unobserve();
    };
  }, [atom]);
  (0, _react.useEffect)(function () {
    atom.fuse({
      actions: config.actions
    });
    setActions(atom.actions);
  }, [config.actions]); // todo, we could allow dynamically swapping actions
  // based on a deps array, for that we'd need to store
  // actions in useState, etc.

  return [state, actions, atom];
}