/**
 * A component local store.
 * A great complement to the global store.
 */
import { useState, useMemo, useEffect } from 'react';
import { createStore } from '../core';
export function useStore(setup, deps) {
  var config = useMemo(function () {
    if (typeof setup === 'function') {
      return setup();
    } else {
      return null;
    }
  }, deps || []);

  var _useState = useState(function () {
    return config ? createStore(config) : setup;
  }),
      atom = _useState[0];

  var _useState2 = useState(function () {
    return atom.actions;
  }),
      actions = _useState2[0],
      setActions = _useState2[1];

  var _useState3 = useState(function () {
    return atom.get();
  }),
      state = _useState3[0],
      setState = _useState3[1];

  useEffect(function () {
    var didUnmount = false;
    var unobserve = atom.observe(function () {
      if (!didUnmount) {
        setState(atom.get());
      }
    });
    return function () {
      didUnmount = true;
      unobserve();
    };
  }, [atom]);
  useEffect(function () {
    atom.fuse({
      actions: config.actions
    });
    setActions(atom.actions);
  }, [config.actions]); // todo, we could allow dynamically swapping actions
  // based on a deps array, for that we'd need to store
  // actions in useState, etc.

  return [state, actions, atom];
}